-------------------------------------------------
--  RESGITERS MODULE
--  Dependencies: none
--  By: Simon Foucher
-------------------------------------------------


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;


ENTITY ID_WB_STAGE IS

      generic (
 		log_file:       string  := "REGISTERS.log"
               );

      PORT(
		-- Control Signals
		RegDst		: in std_logic;	-- Controls the write register access (STAGE ID)
		MemtoReg	: in std_logic;	-- Selects ALU/Memory data (STAGE WB)
		RegWrite	: in std_logic;	-- Sends a write command (STAGE WB)

		Reset		: in std_logic;	-- Resets the system and formats registers

		-- Data Input
		Instruction	: in std_logic_vector(25 downto 0);
		FromMemDataReg	: in std_logic_vector(63 downto 0);		-- Data from Memory
		FromALUDataReg	: in std_logic_vector(63 downto 0);		-- Data from ALU
		
		WriteReg_IN		: in std_logic_vector(4 downto 0);		-- Delayed to WB stage


		-- Data Output
		ReadDataA	: out std_logic_vector(63 downto 0);
		ReadDataB	: out std_logic_vector(63 downto 0);
		SignExtended: out std_logic_vector(31 downto 0);
		WriteReg_OUT: out std_logic_vector(4 downto 0));		-- Decoded in ID then passed on to be delayed to WB



END ID_WB_STAGE;


ARCHITECTURE logic OF ID_WB_STAGE IS


-- Register cells
SIGNAL Reg_0x00 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x01 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x02 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x03 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x04 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x05 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x06 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x07 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x08 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x09 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x0a : std_logic_vector (63 downto 0);
SIGNAL Reg_0x0b : std_logic_vector (63 downto 0);
SIGNAL Reg_0x0c : std_logic_vector (63 downto 0);
SIGNAL Reg_0x0d : std_logic_vector (63 downto 0);
SIGNAL Reg_0x0e : std_logic_vector (63 downto 0);
SIGNAL Reg_0x0f : std_logic_vector (63 downto 0);
SIGNAL Reg_0x10 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x11 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x12 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x13 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x14 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x15 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x16 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x17 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x18 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x19 : std_logic_vector (63 downto 0);
SIGNAL Reg_0x1a : std_logic_vector (63 downto 0);
SIGNAL Reg_0x1b : std_logic_vector (63 downto 0);
SIGNAL Reg_0x1c : std_logic_vector (63 downto 0);
SIGNAL Reg_0x1d : std_logic_vector (63 downto 0);
SIGNAL Reg_0x1e : std_logic_vector (63 downto 0);
SIGNAL Reg_0x1f : std_logic_vector (63 downto 0);


-- Intermediate Data Signals
SIGNAL WriteData: std_logic_vector(63 downto 0);



BEGIN
   
-- Write Register Mux Selects rt od rd field of instruction
WITH  RegDst SELECT
	WriteReg_OUT <=	Instruction(20 downto 16) WHEN '0',		-- Selects rt
					Instruction(15 downto 11) WHEN '1',		-- Selects rs
					"00000" WHEN OTHERS;




-- Write Data Mux Selects between Memory and ALU DATA
WITH  MemtoReg SELECT
	WriteData <=	FromALUDataReg WHEN '0',		-- Selects Data from the ALU
					FromMemDataReg WHEN '1',		-- Selects Data from Memory
					x"0000000000000000" WHEN OTHERS;



-- Sign extend branch address and sll2
SignExtended <= 	Instruction(15)&Instruction(15)&Instruction(15)&Instruction(15)&
					Instruction(15)&Instruction(15)&Instruction(15)&Instruction(15)&
					Instruction(15)&Instruction(15)&Instruction(15)&Instruction(15)&
					Instruction(15)&Instruction(15)&Instruction(15)&Instruction(15)&
					Instruction(15 downto 0);




RW_Reristers : PROCESS(Reset, RegWrite, MemtoReg, RegDst)
    
BEGIN

	IF Reset = '1' THEN
		Reg_0x00 <= x"0000000000000000";
		Reg_0x01 <= x"0000000000000000";
		Reg_0x02 <= x"0000000000000000";
		Reg_0x03 <= x"0000000000000000";
		Reg_0x04 <= x"0000000000000000";
		Reg_0x05 <= x"0000000000000000";
		Reg_0x06 <= x"0000000000000000";
		Reg_0x07 <= x"0000000000000000";
		Reg_0x08 <= x"0000000000000000";
		Reg_0x09 <= x"0000000000000000";
		Reg_0x0a <= x"0000000000000000";
		Reg_0x0b <= x"0000000000000000";
		Reg_0x0c <= x"0000000000000000";
		Reg_0x0d <= x"0000000000000000";
		Reg_0x0e <= x"0000000000000000";
		Reg_0x0f <= x"0000000000000000";
		Reg_0x10 <= x"0000000000000000";
		Reg_0x11 <= x"0000000000000000";
		Reg_0x12 <= x"0000000000000000";
		Reg_0x13 <= x"0000000000000000";
		Reg_0x14 <= x"0000000000000000";
		Reg_0x15 <= x"0000000000000000";
		Reg_0x16 <= x"0000000000000000";
		Reg_0x17 <= x"0000000000000000";
		Reg_0x18 <= x"0000000000000000";
		Reg_0x19 <= x"0000000000000000";
		Reg_0x1a <= x"0000000000000000";
		Reg_0x1b <= x"0000000000000000";
		Reg_0x1c <= x"0000000000000000";
		Reg_0x1d <= x"0000000000000000";
		Reg_0x1e <= x"0000000000000000";
		Reg_0x1f <= x"0000000000000000";




	ELSE
	-- If we are not reseting; First write Data, then automatically read Reg A and Reg B
		IF RegWrite = '1' THEN
			CASE WriteReg_IN IS
				-- NOTE, R0 Forced to 0
				WHEN "00001" => Reg_0x01 <= WriteData;
				WHEN "00010" => Reg_0x02 <= WriteData;
				WHEN "00011" => Reg_0x03 <= WriteData;
				WHEN "00100" => Reg_0x04 <= WriteData;
				WHEN "00101" => Reg_0x05 <= WriteData;
				WHEN "00110" => Reg_0x06 <= WriteData;
				WHEN "00111" => Reg_0x07 <= WriteData;
				WHEN "01000" => Reg_0x08 <= WriteData;
				WHEN "01001" => Reg_0x09 <= WriteData;
				WHEN "01010" => Reg_0x0a <= WriteData;
				WHEN "01011" => Reg_0x0b <= WriteData;
				WHEN "01100" => Reg_0x0c <= WriteData;
				WHEN "01101" => Reg_0x0d <= WriteData;
				WHEN "01110" => Reg_0x0e <= WriteData;
				WHEN "01111" => Reg_0x0f <= WriteData;
				WHEN "10000" => Reg_0x10 <= WriteData;
				WHEN "10001" => Reg_0x11 <= WriteData;
				WHEN "10010" => Reg_0x12 <= WriteData;
				WHEN "10011" => Reg_0x13 <= WriteData;
				WHEN "10100" => Reg_0x14 <= WriteData;
				WHEN "10101" => Reg_0x15 <= WriteData;
				WHEN "10110" => Reg_0x16 <= WriteData;
				WHEN "10111" => Reg_0x17 <= WriteData;
				WHEN "11000" => Reg_0x18 <= WriteData;
				WHEN "11001" => Reg_0x19 <= WriteData;
				WHEN "11010" => Reg_0x1a <= WriteData;
				WHEN "11011" => Reg_0x1b <= WriteData;
				WHEN "11100" => Reg_0x1c <= WriteData;
				WHEN "11101" => Reg_0x1d <= WriteData;
				WHEN "11110" => Reg_0x1e <= WriteData;
				WHEN "11111" => Reg_0x1f <= WriteData;
		 		WHEN OTHERS =>
			END CASE;
		END IF;		-- End if we need to write to register



	CASE Instruction(25 downto 21) IS
		-- Register A
            WHEN "00000" => ReadDataA <= Reg_0x00;
			WHEN "00001" => ReadDataA <= Reg_0x01;
			WHEN "00010" => ReadDataA <= Reg_0x02;
			WHEN "00011" => ReadDataA <= Reg_0x03;
			WHEN "00100" => ReadDataA <= Reg_0x04;
			WHEN "00101" => ReadDataA <= Reg_0x05;
			WHEN "00110" => ReadDataA <= Reg_0x06;
			WHEN "00111" => ReadDataA <= Reg_0x07;
			WHEN "01000" => ReadDataA <= Reg_0x08;
			WHEN "01001" => ReadDataA <= Reg_0x09;
			WHEN "01010" => ReadDataA <= Reg_0x0a;
			WHEN "01011" => ReadDataA <= Reg_0x0b;
			WHEN "01100" => ReadDataA <= Reg_0x0c;
			WHEN "01101" => ReadDataA <= Reg_0x0d;
			WHEN "01110" => ReadDataA <= Reg_0x0e;
			WHEN "01111" => ReadDataA <= Reg_0x0f;
			WHEN "10000" => ReadDataA <= Reg_0x10;
			WHEN "10001" => ReadDataA <= Reg_0x11;
			WHEN "10010" => ReadDataA <= Reg_0x12;
			WHEN "10011" => ReadDataA <= Reg_0x13;
			WHEN "10100" => ReadDataA <= Reg_0x14;
			WHEN "10101" => ReadDataA <= Reg_0x15;
			WHEN "10110" => ReadDataA <= Reg_0x16;
			WHEN "10111" => ReadDataA <= Reg_0x17;
			WHEN "11000" => ReadDataA <= Reg_0x18;
			WHEN "11001" => ReadDataA <= Reg_0x19;
			WHEN "11010" => ReadDataA <= Reg_0x1a;
			WHEN "11011" => ReadDataA <= Reg_0x1b;
			WHEN "11100" => ReadDataA <= Reg_0x1c;
			WHEN "11101" => ReadDataA <= Reg_0x1d;
			WHEN "11110" => ReadDataA <= Reg_0x1e;
			WHEN "11111" => ReadDataA <= Reg_0x1f;
	        WHEN OTHERS => ReadDataA <= x"ffffffffffffffff";       
   END CASE;

	CASE Instruction(20 downto 16) IS
		-- Register B
            WHEN "00000" => ReadDataB <= Reg_0x00;
			WHEN "00001" => ReadDataB <= Reg_0x01;
			WHEN "00010" => ReadDataB <= Reg_0x02;
			WHEN "00011" => ReadDataB <= Reg_0x03;
			WHEN "00100" => ReadDataB <= Reg_0x04;
			WHEN "00101" => ReadDataB <= Reg_0x05;
			WHEN "00110" => ReadDataB <= Reg_0x06;
			WHEN "00111" => ReadDataB <= Reg_0x07;
			WHEN "01000" => ReadDataB <= Reg_0x08;
			WHEN "01001" => ReadDataB <= Reg_0x09;
			WHEN "01010" => ReadDataB <= Reg_0x0a;
			WHEN "01011" => ReadDataB <= Reg_0x0b;
			WHEN "01100" => ReadDataB <= Reg_0x0c;
			WHEN "01101" => ReadDataB <= Reg_0x0d;
			WHEN "01110" => ReadDataB <= Reg_0x0e;
			WHEN "01111" => ReadDataB <= Reg_0x0f;
			WHEN "10000" => ReadDataB <= Reg_0x10;
			WHEN "10001" => ReadDataB <= Reg_0x11;
			WHEN "10010" => ReadDataB <= Reg_0x12;
			WHEN "10011" => ReadDataB <= Reg_0x13;
			WHEN "10100" => ReadDataB <= Reg_0x14;
			WHEN "10101" => ReadDataB <= Reg_0x15;
			WHEN "10110" => ReadDataB <= Reg_0x16;
			WHEN "10111" => ReadDataB <= Reg_0x17;
			WHEN "11000" => ReadDataB <= Reg_0x18;
			WHEN "11001" => ReadDataB <= Reg_0x19;
			WHEN "11010" => ReadDataB <= Reg_0x1a;
			WHEN "11011" => ReadDataB <= Reg_0x1b;
			WHEN "11100" => ReadDataB <= Reg_0x1c;
			WHEN "11101" => ReadDataB <= Reg_0x1d;
			WHEN "11110" => ReadDataB <= Reg_0x1e;
			WHEN "11111" => ReadDataB <= Reg_0x1f;
        WHEN OTHERS => ReadDataB <= x"ffffffffffffffff";      
   END CASE;

    
END IF; --End If reset

END PROCESS;
     
    
END logic;
