------------------------------------------------------------------------
-- Filename: CONTROLLER.vhd
-- Title: Implementation of the controller of the MIPS pipeline 
-- Author: Shajib
-- Date: February 2009
------------------------------------------------------------------------

library ieee;
use	ieee.std_logic_1164.all;
use	ieee.std_logic_arith.all;

ENTITY PIPE_CONTROLLER IS
      PORT(
      
      		-- Control Input
		INSTRUCTION : in std_logic_vector(31 downto 0);
		
		-- Input to PC write logic
		HAZARD : in std_logic;
		-- Control Signals		
		
		-- Execution Address Calculation Stage Lines
		RegDst : out std_logic;	-- Controls the write register access
		ALUOp : out std_logic; -- Controls dependence on funct field
		ALUSrcB : out std_logic; -- Controls where second ALU operand comes from
		
		-- Memory Access Stage Lines
		MemRead : out std_logic; -- Fills memory data output when aserted
		MemWrite : out std_logic; -- Writes data input to memory when asserted
		
		
		-- Write-Back Stage Lines
		RegWrite : out std_logic; -- Sends a write command
		MemtoReg	: out std_logic;	-- Selects ALU/Memory data
		
		HAZARD_CTRL : out std_logic); -- Notifies PC and Pipeline registers of hazard



END PIPE_CONTROLLER;

ARCHITECTURE logic OF PIPE_CONTROLLER IS

BEGIN

  PROCESS (INSTRUCTION(31 downto 26), HAZARD)
  BEGIN

      IF HAZARD = '1' THEN
      	HAZARD_CTRL <= '1';
      ELSE
      	HAZARD_CTRL <= '0';
      END IF;
      
      CASE INSTRUCTION(31 downto 26) IS

        WHEN "110111" => -- LD Instruction
		-- Execution
        		RegDst <= '0';
        		ALUOp <= '0';
		ALUSrcB <= '1';

        		-- Memory
        		MemRead <= '1';
        		MemWrite <= '0';
        		
        		-- Write-Back
        		RegWrite <= '1';
        		MemtoReg <= '1';

        WHEN "111111" => -- SD Instruction
		-- Execution
        		RegDst <= '0'; -- Don't Care
        		ALUOp <= '0';
		ALUSrcB <= '1';

        		-- Memory
        		MemRead <= '0';
        		MemWrite <= '1';
        		
        		-- Write-Back
        		RegWrite <= '0';
        		MemtoReg <= '0'; -- Don't Care
        
        WHEN "011000" => -- DADDI Instruction
        		-- Execution
        		RegDst <= '0';
        		ALUOp <= '0';
		ALUSrcB <= '1';

        		-- Memory
        		MemRead <= '0';
        		MemWrite <= '0';
        		
        		-- Write-Back
        		RegWrite <= '1';
        		MemtoReg <= '0';

        WHEN "011001" => -- DADDIU (in assignment sheet "DADDUI")
		-- Execution
        		RegDst <= '0';
        		ALUOp <= '0';
		ALUSrcB <= '1';

        		-- Memory
        		MemRead <= '0';
        		MemWrite <= '0';
        		
        		-- Write-Back
        		RegWrite <= '1';
        		MemtoReg <= '0';
        		
        WHEN "001100" => -- ANDI
		-- Execution
        		RegDst <= '0';
        		ALUOp <= '0';
		ALUSrcB <= '1';

        		-- Memory
        		MemRead <= '0';
        		MemWrite <= '0';
        		
        		-- Write-Back
        		RegWrite <= '1';
        		MemtoReg <= '0';
        
        WHEN "001101" => -- ORI
        		-- Execution
        		RegDst <= '0';
        		ALUOp <= '0';
		ALUSrcB <= '1';

        		-- Memory
        		MemRead <= '0';
        		MemWrite <= '0';
        		
        		-- Write-Back
        		RegWrite <= '1';
        		MemtoReg <= '0';
        
        WHEN "001110" => -- XORI
		-- Execution
        		RegDst <= '0';
        		ALUOp <= '0';
		ALUSrcB <= '1';

        		-- Memory
        		MemRead <= '0';
        		MemWrite <= '0';
        		
        		-- Write-Back
        		RegWrite <= '1';
        		MemtoReg <= '0';
        
        WHEN "011010" => -- DSUBI
		-- Execution
        		RegDst <= '0';
        		ALUOp <= '0';
		ALUSrcB <= '1';

        		-- Memory
        		MemRead <= '0';
        		MemWrite <= '0';
        		
        		-- Write-Back
        		RegWrite <= '1';
        		MemtoReg <= '0';
        		
        WHEN "011011" => -- DSUBIU
		-- Execution
        		RegDst <= '0';
        		ALUOp <= '0';
		ALUSrcB <= '1';

        		-- Memory
        		MemRead <= '0';
        		MemWrite <= '0';
        		
        		-- Write-Back
        		RegWrite <= '1';
        		MemtoReg <= '0';       
        
        WHEN OTHERS => -- NOP, DADD, DADDU, DSUB, DSUBU, AND, OR, XOR, DSLL, DSRL Instruction
        		-- Execution
        		RegDst <= '1';
        		ALUOp <= '1';
		ALUSrcB <= '0';

        		-- Memory
        		MemRead <= '0';
        		MemWrite <= '0';
        		
        		-- Write-Back
        		RegWrite <= '1';
        		MemtoReg <= '0';
        		
      END CASE;

  END PROCESS;   
    
END logic;



