------------------------------------------------------------------------
-- Filename: REG_EX_MEM.vhd
-- Title: Register between stages EX and MEM
-- Author: Simon Foucher
-- Date: February 2009
------------------------------------------------------------------------

library ieee;
use	ieee.std_logic_1164.all;
use	ieee.std_logic_arith.all;

ENTITY REG_EX_MEM IS
      PORT(
      
		CLK	: in std_logic;
		RST : in std_logic;
		
		-- DATA PORTS
		ALUResult_IN	: in std_logic_vector(63 downto 0);
		ReadRegB_IN		: in std_logic_vector(63 downto 0); 
		WriteReg_IN		: in std_logic_vector(4 downto 0); 

		ALUResult_OUT	: out std_logic_vector(63 downto 0);
		ReadRegB_OUT	: out std_logic_vector(63 downto 0); 
		WriteReg_OUT	: out std_logic_vector(4 downto 0);	-- To Be passed on to WB stage


		-- CONTROL PORTS
		-- For MEM STAGE
	    MemRead_IN     : in std_logic;
  	    MemWrite_IN    : in std_logic;
  	    MemRead_OUT    : out std_logic;
  	    MemWrite_OUT   : out std_logic;
		
		-- For WB STAGE
		MemtoReg_IN		: in std_logic;	
		RegWrite_IN		: in std_logic;	
		MemtoReg_OUT	: out std_logic;	
		RegWrite_OUT	: out std_logic;
		
		-- Instruction Carry forward for hazard detection
		INSTRUCTION_IN : in std_logic_vector(31 downto 0);
		INSTRUCTION_OUT : out std_logic_vector(31 downto 0));	


END REG_EX_MEM;











ARCHITECTURE logic OF REG_EX_MEM IS
BEGIN



PROCESS(CLK, RST)
BEGIN

IF rising_edge(CLK) THEN
	
	IF RST = '1' THEN
	
		ALUResult_OUT	<= x"0000000000000000";
		ReadRegB_OUT	<= x"0000000000000000";
		WriteReg_OUT	<= "00000";
	

  	    MemRead_OUT 	<= '0';
  	    MemWrite_OUT 	<= '0';

		MemtoReg_OUT 	<= '0';
		RegWrite_OUT 	<= '0';
		
		INSTRUCTION_OUT <= x"00000000";
	
	ELSE
		ALUResult_OUT	<= ALUResult_IN;
		ReadRegB_OUT	<= ReadRegB_IN;
		WriteReg_OUT	<= WriteReg_IN;

  	    MemRead_OUT 	<= MemRead_IN;
  	    MemWrite_OUT 	<= MemWrite_IN;

		MemtoReg_OUT 	<= MemtoReg_IN;
		RegWrite_OUT 	<= RegWrite_IN;
		
		INSTRUCTION_OUT <= INSTRUCTION_IN;
	
	END IF;
	
END IF;


END PROCESS;

END logic;
