------------------------------------------------------------------------
-- Filename: REG_ID_EX.vhd
-- Title: Register between stages ID and EX
-- Author: Simon Foucher
-- Date: February 2009
------------------------------------------------------------------------

library ieee;
use	ieee.std_logic_1164.all;
use	ieee.std_logic_arith.all;

ENTITY REG_ID_EX IS
      PORT(
      
		CLK	: in std_logic;
		RST : in std_logic;
		HAZARD : in std_logic;
		
		-- DATA PORTS 
		Data_RegA_IN	: in std_logic_vector(63 downto 0);
		Data_RegB_IN	: in std_logic_vector(63 downto 0);
		Extended_IN		: in std_logic_vector(31 downto 0);
		WriteReg_IN		: in std_logic_vector(4 downto 0); 

		
		Data_RegA_OUT	: out std_logic_vector(63 downto 0);
		Data_RegB_OUT	: out std_logic_vector(63 downto 0);
		Extended_OUT	: out std_logic_vector(31 downto 0);-- To be passed on to WB stage
		WriteReg_OUT	: out std_logic_vector(4 downto 0);	-- To Be passed on to WB stage


		-- CONTROL PORTS
		-- For EX STAGE
		ALU_OP_IN		: in std_logic;
		ALU_Source_IN	: in std_logic;
		ALU_OP_OUT		: out std_logic;
		ALU_Source_OUT	: out std_logic;
		
		-- For MEM STAGE
	    MemRead_IN     : in std_logic;
  	    MemWrite_IN    : in std_logic;
  	    MemRead_OUT    : out std_logic;
  	    MemWrite_OUT   : out std_logic;
		
		-- For WB STAGE
		MemtoReg_IN		: in std_logic;	
		RegWrite_IN		: in std_logic;	
		MemtoReg_OUT	: out std_logic;	
		RegWrite_OUT	: out std_logic;
		
		-- Instruction Carry forward for hazard detection
		INSTRUCTION_IN : in std_logic_vector(31 downto 0);
		INSTRUCTION_OUT : out std_logic_vector(31 downto 0));	


END REG_ID_EX;











ARCHITECTURE logic OF REG_ID_EX IS
BEGIN



PROCESS(CLK, RST)
BEGIN

IF rising_edge(CLK) THEN
	
	IF RST = '1' OR HAZARD = '1' THEN
	-- If hazard, fire a NOP
		Data_RegA_OUT 	<= x"0000000000000000";
		Data_RegB_OUT 	<= x"0000000000000000";
		Extended_OUT 	<= x"00000000";
		WriteReg_OUT 	<= "00000";

		ALU_OP_OUT 		<= '0';
		ALU_Source_OUT 	<= '0';
		
  	    MemRead_OUT 	<= '0';
  	    MemWrite_OUT 	<= '0';

		MemtoReg_OUT 	<= '0';
		RegWrite_OUT 	<= '0';
		
		INSTRUCTION_OUT <= x"00000000";
	
	-- If hazard = '1', register in memory mode
	ELSIF HAZARD = '0' THEN
		Data_RegA_OUT 	<= Data_RegA_IN;
		Data_RegB_OUT 	<= Data_RegB_IN;
		Extended_OUT 	<= Extended_IN;
		WriteReg_OUT 	<= WriteReg_IN;

		ALU_OP_OUT 		<= ALU_OP_IN;
		ALU_Source_OUT 	<= ALU_Source_IN;
		
  	    MemRead_OUT 	<= MemRead_IN;
  	    MemWrite_OUT 	<= MemWrite_IN;

		MemtoReg_OUT 	<= MemtoReg_IN;
		RegWrite_OUT 	<= RegWrite_IN;
		
		INSTRUCTION_OUT <= INSTRUCTION_IN;
		

		
	END IF;
	
END IF;


END PROCESS;

END logic;
