------------------------------------------------------------------------
-- Filename: REG_MEM_WB.vhd
-- Title: Register between stages EX and MEM
-- Author: Simon Foucher
-- Date: February 2009
------------------------------------------------------------------------

library ieee;
use	ieee.std_logic_1164.all;



ENTITY REG_MEM_WB IS
      PORT(
      
		CLK	: in std_logic;
		RST : in std_logic;
		
		-- DATA PORTS
		MemData_IN		: in std_logic_vector(63 downto 0); 
		ALUResult_IN	: in std_logic_vector(63 downto 0);
		WriteReg_IN		: in std_logic_vector(4 downto 0); 
		
		MemData_OUT		: out std_logic_vector(63 downto 0); 
		ALUResult_OUT	: out std_logic_vector(63 downto 0);
		WriteReg_OUT	: out std_logic_vector(4 downto 0);


		-- CONTROL PORTS
		-- For WB STAGE
		MemtoReg_IN		: in std_logic;	
		RegWrite_IN		: in std_logic;	
		MemtoReg_OUT	: out std_logic;	
		RegWrite_OUT	: out std_logic;
		
		-- Instruction Carry forward for hazard detection
		INSTRUCTION_IN : in std_logic_vector(31 downto 0);
		INSTRUCTION_OUT : out std_logic_vector(31 downto 0));	
		
		


END REG_MEM_WB;




ARCHITECTURE logic OF REG_MEM_WB IS
BEGIN



PROCESS(CLK, RST)
BEGIN

IF rising_edge(CLK) THEN
	
	IF RST = '1' THEN
		MemData_OUT		<= x"0000000000000000";
		ALUResult_OUT	<= x"0000000000000000";
		WriteReg_OUT	<= "00000";
	
		MemtoReg_OUT 	<= '0';
		RegWrite_OUT 	<= '0';

		INSTRUCTION_OUT <= x"00000000";
			
	ELSE
		MemData_OUT		<= MemData_IN;
		ALUResult_OUT	<= ALUResult_IN;
		WriteReg_OUT	<= WriteReg_IN;

		MemtoReg_OUT 	<= MemtoReg_IN;
		RegWrite_OUT 	<= RegWrite_IN;
		
		INSTRUCTION_OUT <= INSTRUCTION_IN;

	
	END IF;
	
END IF;


END PROCESS;

END logic;
